#create table
create table users (
	id int unsigned auto_increment, 
	first_name varchar(30) null, 
	last_name varchar(30) null, 
	age int(3) unsigned default 1,

	primary key(id),
	key (first_name),
	key (last_name)
);

#CRUD
#add record
insert into users 
	(first_name, last_name, age) 
	values 
	('John','Banner',30),
	('Mary','Jane',24),
	('Oprah','Winfrey',60);

#read all row from table
select * from users;

#read specific columns
select first_name, last_name from users;

#read specific record by id
select * from users where id = 2;

#read specific columns by specific id
select first_name, last_name from users where id = 3;

#update a row
update users set age = 100, first_name = 'Peter' where id = 3;

#delete a row
delete from users where id = 3;

#change auto increment value
alter table users auto_increment = 3;

#order desc or asc
select * from users order by id desc;
select * from users order by id asc;
select * from users order by age asc;

#order by more than column
select * from users order by id desc, first_name asc;

#limit results
select * from users limit 2;
select * from users order by id desc limit 2;

#offset for pagination
#formula: offset = (page_number - 1) * limit
select * from users order by id asc limit 2 offset 0;

#show database information
show databases;
show tables in mydb;
show columns in users;
describe users;

#checking for NULL values
select * from users where first_name is null;
select * from users where first_name is not null;

#alias
select first_name as name from users;

#add edit delete columnss
alter table users add test int default 0 not null;
alter table users modify column test varchar(10) not null;
alter table users drop column test;

#aggregate functions
#count() sum() min() max() avg()
select count(*) from users;
select sum(age) from users;
select min(age) from users;
select max(age) from users;
select count(*) as total from users where age > 20;

#group by
select * from users group by age;
select count(*) as number, user_role from users where user_role in('admin','user') group by user_role;

#search using like
select * from users where first_name like '%pr%' or last_name like '%pr%';
select * from users where first_name like '_prah';
select * from users where first_name rlike '[a-z]+';

#create new table
create table customers (
	id int unsigned auto_increment, 
	first_name varchar(30) null, 
	last_name varchar(30) null, 
	age int(3) unsigned default 1,
	company varchar(50) null,

	primary key(id),
	key (first_name),
	key (last_name),
	key (company)
);

#add record
insert into customers 
	(first_name, last_name, age, company) 
	values 
	('Jersey','Shore',30,'Mcdonalds'),
	('Dong','Wong',24,'KFC'),
	('Pete','Jefrey',60,'Home Depo');

#combine tables using union
select * from users union select * from customers;

#joins
select * from users cross join customers on users.id = customers.user_id;
select * from users inner join customers on users.id = customers.user_id;
select * from users left join customers on users.id = customers.user_id;

select * from users as u join customers as c on u.id = c.user_id where c.id = 1;
select u.id,u.first_name,u.last_name,c.first_name as customer_name  from users as u join customers as c on u.id = c.user_id where c.id = 1;

select 
	u.id,
	concat_ws(' ',u.first_name,u.last_name) as 'user name',
	concat_ws(' ',c.first_name,c.last_name) as 'customer name' 
	from users as u 
	join customers as c 
	on u.id = c.user_id 
	where c.id = 1;

#sub queries
select * from users where id = (select user_id from customers where first_name = 'Jersey');
select * from users where id in (select user_id from customers where user_id > 0);

select *,(select first_name from users where id = customers.user_id) as user from customers where id = 1;

#having
select age,count(*) as total from users group by age having total > 1;


#dates
#datetime timestamp now time day week month year interval
select now();
select * from users where time(created) = '14:59:21';
select * from users where month(created) = '8' and year(created) = '2023';

#between
#selecting a range
select * from users where date(created) between '2023-10-20' and '2023-10-20';
select * from users where day(created) between '20' and '20';



